@@ -8,12 +8,8 @@ import datetime
 import hashlib
 
 from functools import lru_cache
-from typing import Any, Dict, Union
+from typing import Any, Dict, NamedTuple
 
-import attr
-from deprecated import deprecated
-
-from .collections import ImmutableDict
 from .exceptions import ValidationError
 from .fields.hashes import validate_sha1
 from .hashutil import hash_git_data, hash_to_hex, MultiHash
@@ -26,18 +22,11 @@ RELEASE = "release"
 DIRECTORY = "directory"
 CONTENT = "content"
 
-SWHID_NAMESPACE = "swh"
-SWHID_VERSION = 1
-SWHID_TYPES = ["ori", "snp", "rel", "rev", "dir", "cnt"]
-SWHID_SEP = ":"
-SWHID_CTXT_SEP = ";"
-
-# deprecated variables
-PID_NAMESPACE = SWHID_NAMESPACE
-PID_VERSION = SWHID_VERSION
-PID_TYPES = SWHID_TYPES
-PID_SEP = SWHID_SEP
-PID_CTXT_SEP = SWHID_CTXT_SEP
+PID_NAMESPACE = "swh"
+PID_VERSION = 1
+PID_TYPES = ["ori", "snp", "rel", "rev", "dir", "cnt"]
+PID_SEP = ":"
+PID_CTXT_SEP = ";"
 
 
 @lru_cache()
@@ -306,8 +295,6 @@ def normalize_timestamp(time_representation):
         offset = time_representation["offset"]
         if "negative_utc" in time_representation:
             negative_utc = time_representation["negative_utc"]
-        if negative_utc is None:
-            negative_utc = False
     elif isinstance(time_representation, datetime.datetime):
         seconds = int(time_representation.timestamp())
         microseconds = time_representation.microsecond
@@ -428,7 +415,7 @@ def revision_identifier(revision):
     - author_date
     - committer
     - committer_date
-    - extra_headers or metadata -> extra_headers
+    - metadata -> extra_headers
     - message
 
     A revision's identifier is the 'git'-checksum of a commit manifest
@@ -488,13 +475,22 @@ def revision_identifier(revision):
     )
 
     # Handle extra headers
-    metadata = revision.get("metadata") or {}
-    extra_headers = revision.get("extra_headers", ())
-    if not extra_headers and "extra_headers" in metadata:
-        extra_headers = metadata["extra_headers"]
+    metadata = revision.get("metadata")
+    if not metadata:
+        metadata = {}
 
-    for key, value in extra_headers:
-        components.extend([key, b" ", escape_newlines(value), b"\n"])
+    for key, value in metadata.get("extra_headers", []):
+
+        # Integer values: decimal representation
+        if isinstance(value, int):
+            value = str(value).encode("utf-8")
+
+        # Unicode string values: utf-8 encoding
+        if isinstance(value, str):
+            value = value.encode("utf-8")
+
+        # encode the key to utf-8
+        components.extend([key.encode("utf-8"), b" ", escape_newlines(value), b"\n"])
 
     if revision["message"] is not None:
         components.extend([b"\n", revision["message"]])
@@ -651,24 +647,37 @@ _object_type_map = {
 }
 
 
-@attr.s(frozen=True)
-class SWHID:
+_PersistentId = NamedTuple(
+    "PersistentId",
+    [
+        ("namespace", str),
+        ("scheme_version", int),
+        ("object_type", str),
+        ("object_id", str),
+        ("metadata", Dict[str, Any]),
+    ],
+)
+
+
+class PersistentId(_PersistentId):
     """
-    Named tuple holding the relevant info associated to a SoftWare Heritage
-    persistent IDentifier (SWHID)
+    Named tuple holding the relevant info associated to a Software Heritage
+    persistent identifier.
 
     Args:
-        namespace (str): the namespace of the identifier, defaults to ``swh``
+        namespace (str): the namespace of the identifier, defaults to 'swh'
         scheme_version (int): the scheme version of the identifier,
             defaults to 1
         object_type (str): the type of object the identifier points to,
-            either ``content``, ``directory``, ``release``, ``revision`` or ``snapshot``
-        object_id (str): object's identifier
+            either 'content', 'directory', 'release', 'revision' or 'snapshot'
+        object_id (dict/bytes/str): object's dict representation or
+            object identifier
         metadata (dict): optional dict filled with metadata related to
             pointed object
 
     Raises:
-        swh.model.exceptions.ValidationError: In case of invalid object type or id
+        swh.model.exceptions.ValidationError: In case of invalid object type
+            or id
 
     Once created, it contains the following attributes:
 
@@ -679,129 +688,94 @@ class SWHID:
         object_id (str): hexadecimal representation of the object hash
         metadata (dict): metadata related to the pointed object
 
-    To get the raw SWHID string from an instance of this named tuple,
-    use the :func:`str` function::
+    To get the raw persistent identifier string from an instance of
+    this named tuple, use the :func:`str` function::
 
-        swhid = SWHID(
+        pid = PersistentId(
             object_type='content',
             object_id='8ff44f081d43176474b267de5451f2c2e88089d0'
         )
-        swhid_str = str(swhid)
+        pid_str = str(pid)
         # 'swh:1:cnt:8ff44f081d43176474b267de5451f2c2e88089d0'
     """
 
-    namespace = attr.ib(type=str, default="swh")
-    scheme_version = attr.ib(type=int, default=1)
-    object_type = attr.ib(type=str, default="")
-    object_id = attr.ib(type=str, converter=hash_to_hex, default="")  # type: ignore
-    metadata = attr.ib(
-        type=ImmutableDict[str, Any], converter=ImmutableDict, default=ImmutableDict()
-    )
+    __slots__ = ()
 
-    @namespace.validator
-    def check_namespace(self, attribute, value):
-        if value != SWHID_NAMESPACE:
-            raise ValidationError(
-                "Wrong format: only supported namespace is '%s'" % SWHID_NAMESPACE
-            )
-
-    @scheme_version.validator
-    def check_scheme_version(self, attribute, value):
-        if value != SWHID_VERSION:
-            raise ValidationError(
-                "Wrong format: only supported version is %d" % SWHID_VERSION
-            )
-
-    @object_type.validator
-    def check_object_type(self, attribute, value):
-        if value not in _object_type_map:
+    def __new__(
+        cls,
+        namespace=PID_NAMESPACE,
+        scheme_version=PID_VERSION,
+        object_type="",
+        object_id="",
+        metadata={},
+    ):
+        o = _object_type_map.get(object_type)
+        if not o:
             raise ValidationError(
                 "Wrong input: Supported types are %s" % (list(_object_type_map.keys()))
             )
+        if namespace != PID_NAMESPACE:
+            raise ValidationError(
+                "Wrong format: only supported namespace is '%s'" % PID_NAMESPACE
+            )
+        if scheme_version != PID_VERSION:
+            raise ValidationError(
+                "Wrong format: only supported version is %d" % PID_VERSION
+            )
+        # internal swh representation resolution
+        if isinstance(object_id, dict):
+            object_id = object_id[o["key_id"]]
+        validate_sha1(object_id)  # can raise if invalid hash
+        object_id = hash_to_hex(object_id)
+        return super(cls, PersistentId).__new__(
+            cls, namespace, scheme_version, object_type, object_id, metadata
+        )
 
-    @object_id.validator
-    def check_object_id(self, attribute, value):
-        validate_sha1(value)  # can raise if invalid hash
-
-    def to_dict(self) -> Dict[str, Any]:
-        return attr.asdict(self)
-
-    def __str__(self) -> str:
+    def __str__(self):
         o = _object_type_map.get(self.object_type)
-        assert o
-        swhid = SWHID_SEP.join(
+        pid = PID_SEP.join(
             [self.namespace, str(self.scheme_version), o["short_name"], self.object_id]
         )
         if self.metadata:
             for k, v in self.metadata.items():
-                swhid += "%s%s=%s" % (SWHID_CTXT_SEP, k, v)
-        return swhid
+                pid += "%s%s=%s" % (PID_CTXT_SEP, k, v)
+        return pid
 
 
-@deprecated("Use swh.model.identifiers.SWHID instead")
-class PersistentId(SWHID):
-    """
-    Named tuple holding the relevant info associated to a SoftWare Heritage
-    persistent IDentifier.
-
-    .. deprecated:: 0.3.8
-        Use :class:`swh.model.identifiers.SWHID` instead
-
-    """
-
-    pass
-
-
-def swhid(
-    object_type: str,
-    object_id: Union[str, Dict[str, Any]],
-    scheme_version: int = 1,
-    metadata: Union[ImmutableDict[str, Any], Dict[str, Any]] = ImmutableDict(),
-) -> str:
-    """Compute :ref:`persistent-identifiers`
+def persistent_identifier(object_type, object_id, scheme_version=1, metadata={}):
+    """Compute :ref:`SWHID <persistent-identifiers>` persistent identifiers.
 
     Args:
-        object_type: object's type, either ``content``, ``directory``,
-            ``release``, ``revision`` or ``snapshot``
-        object_id: object's identifier
-        scheme_version: SWHID scheme version, defaults to 1
-        metadata: metadata related to the pointed object
+        object_type (str): object's type, either 'content', 'directory',
+            'release', 'revision' or 'snapshot'
+        object_id (dict/bytes/str): object's dict representation or object
+            identifier
+        scheme_version (int): persistent identifier scheme version,
+            defaults to 1
+        metadata (dict): metadata related to the pointed object
 
     Raises:
-        swh.model.exceptions.ValidationError: In case of invalid object type or id
+        swh.model.exceptions.ValidationError: In case of invalid object type
+        or id
 
     Returns:
-        the SWHID of the object
+        str: the persistent identifier
 
     """
-    if isinstance(object_id, dict):
-        o = _object_type_map[object_type]
-        object_id = object_id[o["key_id"]]
-    swhid = SWHID(
+    pid = PersistentId(
         scheme_version=scheme_version,
         object_type=object_type,
         object_id=object_id,
-        metadata=metadata,  # type: ignore  # mypy can't properly unify types
+        metadata=metadata,
     )
-    return str(swhid)
+    return str(pid)
 
 
-@deprecated("Use swh.model.identifiers.swhid instead")
-def persistent_identifier(*args, **kwargs) -> str:
-    """Compute :ref:`persistent-identifiers`
-
-    .. deprecated:: 0.3.8
-        Use :func:`swh.model.identifiers.swhid` instead
-
-    """
-    return swhid(*args, **kwargs)
-
-
-def parse_swhid(swhid: str) -> SWHID:
-    """Parse :ref:`persistent-identifiers`.
+def parse_persistent_identifier(persistent_id):
+    """Parse :ref:`SWHID <persistent-identifiers>` persistent identifiers.
 
     Args:
-        swhid (str): A persistent identifier
+        persistent_id (str): A persistent identifier
 
     Raises:
         swh.model.exceptions.ValidationError: in case of:
@@ -814,49 +788,35 @@ def parse_swhid(swhid: str) -> SWHID:
             * invalid hash identifier supplied
 
     Returns:
-        a named tuple holding the parsing result
+        PersistentId: a named tuple holding the parsing result
 
     """
-    # <swhid>;<contextual-information>
-    swhid_parts = swhid.split(SWHID_CTXT_SEP)
-    swhid_data = swhid_parts.pop(0).split(":")
+    # <pid>;<contextual-information>
+    persistent_id_parts = persistent_id.split(PID_CTXT_SEP)
+    pid_data = persistent_id_parts.pop(0).split(":")
 
-    if len(swhid_data) != 4:
+    if len(pid_data) != 4:
         raise ValidationError("Wrong format: There should be 4 mandatory values")
 
     # Checking for parsing errors
-    _ns, _version, _type, _id = swhid_data
+    _ns, _version, _type, _id = pid_data
+    pid_data[1] = int(pid_data[1])
 
     for otype, data in _object_type_map.items():
         if _type == data["short_name"]:
-            _type = otype
+            pid_data[2] = otype
             break
 
     if not _id:
         raise ValidationError("Wrong format: Identifier should be present")
 
-    _metadata = {}
-    for part in swhid_parts:
+    persistent_id_metadata = {}
+    for part in persistent_id_parts:
         try:
             key, val = part.split("=")
-            _metadata[key] = val
+            persistent_id_metadata[key] = val
         except Exception:
             msg = "Contextual data is badly formatted, form key=val expected"
             raise ValidationError(msg)
-    return SWHID(
-        _ns,
-        int(_version),
-        _type,
-        _id,
-        _metadata,  # type: ignore  # mypy can't properly unify types
-    )
-
-
-@deprecated("Use swh.model.identifiers.parse_swhid instead")
-def parse_persistent_identifier(persistent_id: str) -> PersistentId:
-    """Parse :ref:`persistent-identifiers`.
-
-    .. deprecated:: 0.3.8
-        Use :func:`swh.model.identifiers.parse_swhid` instead
-    """
-    return PersistentId(**parse_swhid(persistent_id).to_dict())
+    pid_data.append(persistent_id_metadata)
+    return PersistentId(*pid_data)
