@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2020 The Software Heritage developers
+# Copyright (C) 2018-2019 The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -7,26 +7,21 @@ import datetime
 
 from abc import ABCMeta, abstractmethod
 from enum import Enum
-from hashlib import sha256
-from typing import Any, Dict, Iterable, Optional, Tuple, TypeVar, Union
-from typing_extensions import Final
+from typing import Dict, List, Optional, Union
 
 import attr
 from attrs_strict import type_validator
 import dateutil.parser
 import iso8601
 
-from .collections import ImmutableDict
-from .hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, MultiHash
 from .identifiers import (
     normalize_timestamp,
     directory_identifier,
     revision_identifier,
     release_identifier,
     snapshot_identifier,
-    SWHID,
-    parse_swhid,
 )
+from .hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, MultiHash
 
 
 class MissingData(Exception):
@@ -42,38 +37,20 @@ SHA1_SIZE = 20
 Sha1Git = bytes
 
 
-KT = TypeVar("KT")
-VT = TypeVar("VT")
-
-
-def freeze_optional_dict(
-    d: Union[None, Dict[KT, VT], ImmutableDict[KT, VT]]  # type: ignore
-) -> Optional[ImmutableDict[KT, VT]]:
-    if isinstance(d, dict):
-        return ImmutableDict(d)
-    else:
-        return d
-
-
 def dictify(value):
     "Helper function used by BaseModel.to_dict()"
     if isinstance(value, BaseModel):
         return value.to_dict()
-    elif isinstance(value, SWHID):
-        return str(value)
     elif isinstance(value, Enum):
         return value.value
-    elif isinstance(value, (dict, ImmutableDict)):
+    elif isinstance(value, dict):
         return {k: dictify(v) for k, v in value.items()}
-    elif isinstance(value, tuple):
-        return tuple(dictify(v) for v in value)
+    elif isinstance(value, list):
+        return [dictify(v) for v in value]
     else:
         return value
 
 
-ModelType = TypeVar("ModelType", bound="BaseModel")
-
-
 class BaseModel:
     """Base class for SWH model classes.
 
@@ -91,13 +68,6 @@ class BaseModel:
         recursively builds the corresponding objects."""
         return cls(**d)
 
-    def anonymize(self: ModelType) -> Optional[ModelType]:
-        """Returns an anonymized version of the object, if needed.
-
-        If the object model does not need/support anonymization, returns None.
-        """
-        return None
-
 
 class HashableObject(metaclass=ABCMeta):
     """Mixin to automatically compute object identifier hash when
@@ -120,8 +90,6 @@ class HashableObject(metaclass=ABCMeta):
 class Person(BaseModel):
     """Represents the author/committer of a revision or release."""
 
-    object_type: Final = "person"
-
     fullname = attr.ib(type=bytes, validator=type_validator())
     name = attr.ib(type=Optional[bytes], validator=type_validator())
     email = attr.ib(type=Optional[bytes], validator=type_validator())
@@ -161,21 +129,11 @@ class Person(BaseModel):
 
         return Person(name=name or None, email=email or None, fullname=fullname,)
 
-    def anonymize(self) -> "Person":
-        """Returns an anonymized version of the Person object.
-
-        Anonymization is simply a Person which fullname is the hashed, with unset name
-        or email.
-        """
-        return Person(fullname=sha256(self.fullname).digest(), name=None, email=None,)
-
 
 @attr.s(frozen=True)
 class Timestamp(BaseModel):
     """Represents a naive timestamp from a VCS."""
 
-    object_type: Final = "timestamp"
-
     seconds = attr.ib(type=int, validator=type_validator())
     microseconds = attr.ib(type=int, validator=type_validator())
 
@@ -196,8 +154,6 @@ class Timestamp(BaseModel):
 class TimestampWithTimezone(BaseModel):
     """Represents a TZ-aware timestamp from a VCS."""
 
-    object_type: Final = "timestamp_with_timezone"
-
     timestamp = attr.ib(type=Timestamp, validator=type_validator())
     offset = attr.ib(type=int, validator=type_validator())
     negative_utc = attr.ib(type=bool, validator=type_validator())
@@ -248,23 +204,26 @@ class TimestampWithTimezone(BaseModel):
 class Origin(BaseModel):
     """Represents a software source: a VCS and an URL."""
 
-    object_type: Final = "origin"
-
     url = attr.ib(type=str, validator=type_validator())
 
 
 @attr.s(frozen=True)
 class OriginVisit(BaseModel):
-    """Represents an origin visit with a given type at a given point in time, by a
+    """Represents a visit of an origin at a given point in time, by a
     SWH loader."""
 
-    object_type: Final = "origin_visit"
-
     origin = attr.ib(type=str, validator=type_validator())
     date = attr.ib(type=datetime.datetime, validator=type_validator())
+    status = attr.ib(
+        type=str, validator=attr.validators.in_(["ongoing", "full", "partial"])
+    )
     type = attr.ib(type=str, validator=type_validator())
-    """Should not be set before calling 'origin_visit_add()'."""
+    snapshot = attr.ib(type=Optional[Sha1Git], validator=type_validator())
+    metadata = attr.ib(
+        type=Optional[Dict[str, object]], validator=type_validator(), default=None
+    )
     visit = attr.ib(type=Optional[int], validator=type_validator(), default=None)
+    """Should not be set before calling 'origin_visit_add()'."""
 
     def to_dict(self):
         """Serializes the date as a string and omits the visit id if it is
@@ -274,6 +233,14 @@ class OriginVisit(BaseModel):
             del ov["visit"]
         return ov
 
+    @classmethod
+    def from_dict(cls, d):
+        """Parses the date from a string, and accepts missing visit ids."""
+        if isinstance(d["date"], str):
+            d = d.copy()
+            d["date"] = dateutil.parser.parse(d["date"])
+        return super().from_dict(d)
+
 
 @attr.s(frozen=True)
 class OriginVisitStatus(BaseModel):
@@ -281,22 +248,16 @@ class OriginVisitStatus(BaseModel):
 
     """
 
-    object_type: Final = "origin_visit_status"
-
     origin = attr.ib(type=str, validator=type_validator())
     visit = attr.ib(type=int, validator=type_validator())
 
     date = attr.ib(type=datetime.datetime, validator=type_validator())
     status = attr.ib(
-        type=str,
-        validator=attr.validators.in_(["created", "ongoing", "full", "partial"]),
+        type=str, validator=attr.validators.in_(["ongoing", "full", "partial"])
     )
     snapshot = attr.ib(type=Optional[Sha1Git], validator=type_validator())
     metadata = attr.ib(
-        type=Optional[ImmutableDict[str, object]],
-        validator=type_validator(),
-        converter=freeze_optional_dict,
-        default=None,
+        type=Optional[Dict[str, object]], validator=type_validator(), default=None
     )
 
 
@@ -326,8 +287,6 @@ class ObjectType(Enum):
 class SnapshotBranch(BaseModel):
     """Represents one of the branches of a snapshot."""
 
-    object_type: Final = "snapshot_branch"
-
     target = attr.ib(type=bytes, validator=type_validator())
     target_type = attr.ib(type=TargetType, validator=type_validator())
 
@@ -348,12 +307,8 @@ class SnapshotBranch(BaseModel):
 class Snapshot(BaseModel, HashableObject):
     """Represents the full state of an origin at a given point in time."""
 
-    object_type: Final = "snapshot"
-
     branches = attr.ib(
-        type=ImmutableDict[bytes, Optional[SnapshotBranch]],
-        validator=type_validator(),
-        converter=freeze_optional_dict,
+        type=Dict[bytes, Optional[SnapshotBranch]], validator=type_validator()
     )
     id = attr.ib(type=Sha1Git, validator=type_validator(), default=b"")
 
@@ -365,18 +320,16 @@ class Snapshot(BaseModel, HashableObject):
     def from_dict(cls, d):
         d = d.copy()
         return cls(
-            branches=ImmutableDict(
-                (name, SnapshotBranch.from_dict(branch) if branch else None)
+            branches={
+                name: SnapshotBranch.from_dict(branch) if branch else None
                 for (name, branch) in d.pop("branches").items()
-            ),
+            },
             **d,
         )
 
 
 @attr.s(frozen=True)
 class Release(BaseModel, HashableObject):
-    object_type: Final = "release"
-
     name = attr.ib(type=bytes, validator=type_validator())
     message = attr.ib(type=Optional[bytes], validator=type_validator())
     target = attr.ib(type=Optional[Sha1Git], validator=type_validator())
@@ -387,10 +340,7 @@ class Release(BaseModel, HashableObject):
         type=Optional[TimestampWithTimezone], validator=type_validator(), default=None
     )
     metadata = attr.ib(
-        type=Optional[ImmutableDict[str, object]],
-        validator=type_validator(),
-        converter=freeze_optional_dict,
-        default=None,
+        type=Optional[Dict[str, object]], validator=type_validator(), default=None
     )
     id = attr.ib(type=Sha1Git, validator=type_validator(), default=b"")
 
@@ -419,14 +369,6 @@ class Release(BaseModel, HashableObject):
             d["date"] = TimestampWithTimezone.from_dict(d["date"])
         return cls(target_type=ObjectType(d.pop("target_type")), **d)
 
-    def anonymize(self) -> "Release":
-        """Returns an anonymized version of the Release object.
-
-        Anonymization consists in replacing the author with an anonymized Person object.
-        """
-        author = self.author and self.author.anonymize()
-        return attr.evolve(self, author=author)
-
 
 class RevisionType(Enum):
     GIT = "git"
@@ -436,15 +378,9 @@ class RevisionType(Enum):
     MERCURIAL = "hg"
 
 
-def tuplify_extra_headers(value: Iterable) -> Tuple:
-    return tuple((k, v) for k, v in value)
-
-
 @attr.s(frozen=True)
 class Revision(BaseModel, HashableObject):
-    object_type: Final = "revision"
-
-    message = attr.ib(type=Optional[bytes], validator=type_validator())
+    message = attr.ib(type=bytes, validator=type_validator())
     author = attr.ib(type=Person, validator=type_validator())
     committer = attr.ib(type=Person, validator=type_validator())
     date = attr.ib(type=Optional[TimestampWithTimezone], validator=type_validator())
@@ -455,33 +391,12 @@ class Revision(BaseModel, HashableObject):
     directory = attr.ib(type=Sha1Git, validator=type_validator())
     synthetic = attr.ib(type=bool, validator=type_validator())
     metadata = attr.ib(
-        type=Optional[ImmutableDict[str, object]],
-        validator=type_validator(),
-        converter=freeze_optional_dict,
-        default=None,
+        type=Optional[Dict[str, object]], validator=type_validator(), default=None
+    )
+    parents = attr.ib(
+        type=List[Sha1Git], validator=type_validator(), default=attr.Factory(list)
     )
-    parents = attr.ib(type=Tuple[Sha1Git, ...], validator=type_validator(), default=())
     id = attr.ib(type=Sha1Git, validator=type_validator(), default=b"")
-    extra_headers = attr.ib(
-        type=Tuple[Tuple[bytes, bytes], ...],  # but it makes mypy sad
-        validator=type_validator(),
-        converter=tuplify_extra_headers,  # type: ignore
-        default=(),
-    )
-
-    def __attrs_post_init__(self):
-        super().__attrs_post_init__()
-        # ensure metadata is a deep copy of whatever was given, and if needed
-        # extract extra_headers from there
-        if self.metadata:
-            metadata = self.metadata
-            if not self.extra_headers and "extra_headers" in metadata:
-                (extra_headers, metadata) = metadata.copy_pop("extra_headers")
-                object.__setattr__(
-                    self, "extra_headers", tuplify_extra_headers(extra_headers),
-                )
-                attr.validate(self)
-            object.__setattr__(self, "metadata", metadata)
 
     @staticmethod
     def compute_hash(object_dict):
@@ -504,25 +419,12 @@ class Revision(BaseModel, HashableObject):
             date=date,
             committer_date=committer_date,
             type=RevisionType(d.pop("type")),
-            parents=tuple(d.pop("parents")),  # for BW compat
             **d,
         )
 
-    def anonymize(self) -> "Revision":
-        """Returns an anonymized version of the Revision object.
-
-        Anonymization consists in replacing the author and committer with an anonymized
-        Person object.
-        """
-        return attr.evolve(
-            self, author=self.author.anonymize(), committer=self.committer.anonymize()
-        )
-
 
 @attr.s(frozen=True)
 class DirectoryEntry(BaseModel):
-    object_type: Final = "directory_entry"
-
     name = attr.ib(type=bytes, validator=type_validator())
     type = attr.ib(type=str, validator=attr.validators.in_(["file", "dir", "rev"]))
     target = attr.ib(type=Sha1Git, validator=type_validator())
@@ -532,9 +434,7 @@ class DirectoryEntry(BaseModel):
 
 @attr.s(frozen=True)
 class Directory(BaseModel, HashableObject):
-    object_type: Final = "directory"
-
-    entries = attr.ib(type=Tuple[DirectoryEntry, ...], validator=type_validator())
+    entries = attr.ib(type=List[DirectoryEntry], validator=type_validator())
     id = attr.ib(type=Sha1Git, validator=type_validator(), default=b"")
 
     @staticmethod
@@ -545,10 +445,7 @@ class Directory(BaseModel, HashableObject):
     def from_dict(cls, d):
         d = d.copy()
         return cls(
-            entries=tuple(
-                DirectoryEntry.from_dict(entry) for entry in d.pop("entries")
-            ),
-            **d,
+            entries=[DirectoryEntry.from_dict(entry) for entry in d.pop("entries")], **d
         )
 
 
@@ -590,8 +487,6 @@ class BaseContent(BaseModel):
 
 @attr.s(frozen=True)
 class Content(BaseContent):
-    object_type: Final = "content"
-
     sha1 = attr.ib(type=bytes, validator=type_validator())
     sha1_git = attr.ib(type=Sha1Git, validator=type_validator())
     sha256 = attr.ib(type=bytes, validator=type_validator())
@@ -608,10 +503,7 @@ class Content(BaseContent):
     data = attr.ib(type=Optional[bytes], validator=type_validator(), default=None)
 
     ctime = attr.ib(
-        type=Optional[datetime.datetime],
-        validator=type_validator(),
-        default=None,
-        eq=False,
+        type=Optional[datetime.datetime], validator=type_validator(), default=None
     )
 
     @length.validator
@@ -658,8 +550,6 @@ class Content(BaseContent):
 
 @attr.s(frozen=True)
 class SkippedContent(BaseContent):
-    object_type: Final = "skipped_content"
-
     sha1 = attr.ib(type=Optional[bytes], validator=type_validator())
     sha1_git = attr.ib(type=Optional[Sha1Git], validator=type_validator())
     sha256 = attr.ib(type=Optional[bytes], validator=type_validator())
@@ -673,10 +563,7 @@ class SkippedContent(BaseContent):
     origin = attr.ib(type=Optional[str], validator=type_validator(), default=None)
 
     ctime = attr.ib(
-        type=Optional[datetime.datetime],
-        validator=type_validator(),
-        default=None,
-        eq=False,
+        type=Optional[datetime.datetime], validator=type_validator(), default=None
     )
 
     @reason.validator
@@ -723,264 +610,3 @@ class SkippedContent(BaseContent):
         if d2.pop("data", None) is not None:
             raise ValueError('SkippedContent has no "data" attribute %r' % d)
         return super().from_dict(d2, use_subclass=False)
-
-
-class MetadataAuthorityType(Enum):
-    DEPOSIT_CLIENT = "deposit_client"
-    FORGE = "forge"
-    REGISTRY = "registry"
-
-
-@attr.s(frozen=True)
-class MetadataAuthority(BaseModel):
-    """Represents an entity that provides metadata about an origin or
-    software artifact."""
-
-    type = attr.ib(type=MetadataAuthorityType, validator=type_validator())
-    url = attr.ib(type=str, validator=type_validator())
-    metadata = attr.ib(
-        type=Optional[ImmutableDict[str, Any]],
-        default=None,
-        validator=type_validator(),
-        converter=freeze_optional_dict,
-    )
-
-    def to_dict(self):
-        d = super().to_dict()
-        if d["metadata"] is None:
-            del d["metadata"]
-        return d
-
-    @classmethod
-    def from_dict(cls, d):
-        d["type"] = MetadataAuthorityType(d["type"])
-        return super().from_dict(d)
-
-
-@attr.s(frozen=True)
-class MetadataFetcher(BaseModel):
-    """Represents a software component used to fetch metadata from a metadata
-    authority, and ingest them into the Software Heritage archive."""
-
-    name = attr.ib(type=str, validator=type_validator())
-    version = attr.ib(type=str, validator=type_validator())
-    metadata = attr.ib(
-        type=Optional[ImmutableDict[str, Any]],
-        default=None,
-        validator=type_validator(),
-        converter=freeze_optional_dict,
-    )
-
-    def to_dict(self):
-        d = super().to_dict()
-        if d["metadata"] is None:
-            del d["metadata"]
-        return d
-
-
-class MetadataTargetType(Enum):
-    """The type of object extrinsic metadata refer to."""
-
-    CONTENT = "content"
-    DIRECTORY = "directory"
-    REVISION = "revision"
-    RELEASE = "release"
-    SNAPSHOT = "snapshot"
-    ORIGIN = "origin"
-
-
-@attr.s(frozen=True)
-class RawExtrinsicMetadata(BaseModel):
-    # target object
-    type = attr.ib(type=MetadataTargetType, validator=type_validator())
-    id = attr.ib(type=Union[str, SWHID], validator=type_validator())
-    """URL if type=MetadataTargetType.ORIGIN, else core SWHID"""
-
-    # source
-    discovery_date = attr.ib(type=datetime.datetime, validator=type_validator())
-    authority = attr.ib(type=MetadataAuthority, validator=type_validator())
-    fetcher = attr.ib(type=MetadataFetcher, validator=type_validator())
-
-    # the metadata itself
-    format = attr.ib(type=str, validator=type_validator())
-    metadata = attr.ib(type=bytes, validator=type_validator())
-
-    # context
-    origin = attr.ib(type=Optional[str], default=None, validator=type_validator())
-    visit = attr.ib(type=Optional[int], default=None, validator=type_validator())
-    snapshot = attr.ib(type=Optional[SWHID], default=None, validator=type_validator())
-    release = attr.ib(type=Optional[SWHID], default=None, validator=type_validator())
-    revision = attr.ib(type=Optional[SWHID], default=None, validator=type_validator())
-    path = attr.ib(type=Optional[bytes], default=None, validator=type_validator())
-    directory = attr.ib(type=Optional[SWHID], default=None, validator=type_validator())
-
-    @id.validator
-    def check_id(self, attribute, value):
-        if self.type == MetadataTargetType.ORIGIN:
-            if isinstance(value, SWHID) or value.startswith("swh:"):
-                raise ValueError(
-                    "Got SWHID as id for origin metadata (expected an URL)."
-                )
-        else:
-            self._check_pid(self.type.value, value)
-
-    @origin.validator
-    def check_origin(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (
-            MetadataTargetType.SNAPSHOT,
-            MetadataTargetType.RELEASE,
-            MetadataTargetType.REVISION,
-            MetadataTargetType.DIRECTORY,
-            MetadataTargetType.CONTENT,
-        ):
-            raise ValueError(
-                f"Unexpected 'origin' context for {self.type.value} object: {value}"
-            )
-
-        if value.startswith("swh:"):
-            # Technically this is valid; but:
-            # 1. SWHIDs are URIs, not URLs
-            # 2. if a SWHID gets here, it's very likely to be a mistake
-            #    (and we can remove this check if it turns out there is a
-            #    legitimate use for it).
-            raise ValueError(f"SWHID used as context origin URL: {value}")
-
-    @visit.validator
-    def check_visit(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (
-            MetadataTargetType.SNAPSHOT,
-            MetadataTargetType.RELEASE,
-            MetadataTargetType.REVISION,
-            MetadataTargetType.DIRECTORY,
-            MetadataTargetType.CONTENT,
-        ):
-            raise ValueError(
-                f"Unexpected 'visit' context for {self.type.value} object: {value}"
-            )
-
-        if self.origin is None:
-            raise ValueError("'origin' context must be set if 'visit' is.")
-
-        if value <= 0:
-            raise ValueError("Nonpositive visit id")
-
-    @snapshot.validator
-    def check_snapshot(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (
-            MetadataTargetType.RELEASE,
-            MetadataTargetType.REVISION,
-            MetadataTargetType.DIRECTORY,
-            MetadataTargetType.CONTENT,
-        ):
-            raise ValueError(
-                f"Unexpected 'snapshot' context for {self.type.value} object: {value}"
-            )
-
-        self._check_pid("snapshot", value)
-
-    @release.validator
-    def check_release(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (
-            MetadataTargetType.REVISION,
-            MetadataTargetType.DIRECTORY,
-            MetadataTargetType.CONTENT,
-        ):
-            raise ValueError(
-                f"Unexpected 'release' context for {self.type.value} object: {value}"
-            )
-
-        self._check_pid("release", value)
-
-    @revision.validator
-    def check_revision(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (MetadataTargetType.DIRECTORY, MetadataTargetType.CONTENT,):
-            raise ValueError(
-                f"Unexpected 'revision' context for {self.type.value} object: {value}"
-            )
-
-        self._check_pid("revision", value)
-
-    @path.validator
-    def check_path(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (MetadataTargetType.DIRECTORY, MetadataTargetType.CONTENT,):
-            raise ValueError(
-                f"Unexpected 'path' context for {self.type.value} object: {value}"
-            )
-
-    @directory.validator
-    def check_directory(self, attribute, value):
-        if value is None:
-            return
-
-        if self.type not in (MetadataTargetType.CONTENT,):
-            raise ValueError(
-                f"Unexpected 'directory' context for {self.type.value} object: {value}"
-            )
-
-        self._check_pid("directory", value)
-
-    def _check_pid(self, expected_object_type, pid):
-        if isinstance(pid, str):
-            raise ValueError(f"Expected SWHID, got a string: {pid}")
-
-        if pid.object_type != expected_object_type:
-            raise ValueError(
-                f"Expected SWHID type '{expected_object_type}', "
-                f"got '{pid.object_type}' in {pid}"
-            )
-
-        if pid.metadata:
-            raise ValueError(f"Expected core SWHID, but got: {pid}")
-
-    def to_dict(self):
-        d = super().to_dict()
-        context_keys = (
-            "origin",
-            "visit",
-            "snapshot",
-            "release",
-            "revision",
-            "directory",
-            "path",
-        )
-        for context_key in context_keys:
-            if d[context_key] is None:
-                del d[context_key]
-        return d
-
-    @classmethod
-    def from_dict(cls, d):
-        d = {
-            **d,
-            "type": MetadataTargetType(d["type"]),
-            "authority": MetadataAuthority.from_dict(d["authority"]),
-            "fetcher": MetadataFetcher.from_dict(d["fetcher"]),
-        }
-
-        if d["type"] != MetadataTargetType.ORIGIN:
-            d["id"] = parse_swhid(d["id"])
-
-        swhid_keys = ("snapshot", "release", "revision", "directory")
-        for swhid_key in swhid_keys:
-            if d.get(swhid_key):
-                d[swhid_key] = parse_swhid(d[swhid_key])
-
-        return super().from_dict(d)
