@@ -5,7 +5,6 @@
 
 import binascii
 import datetime
-import pytest
 import unittest
 
 from swh.model import hashutil, identifiers
@@ -17,8 +16,7 @@ from swh.model.identifiers import (
     RELEASE,
     REVISION,
     SNAPSHOT,
-    SWHID,
-    normalize_timestamp,
+    PersistentId,
 )
 
 
@@ -370,10 +368,12 @@ dg1KdHOa34shrKDaOVzW
                 2015, 7, 12, 15, 10, 30, tzinfo=linus_tz
             ),
             "message": b"Linux 4.2-rc2\n",
-            "extra_headers": (
-                (b"svn-repo-uuid", b"046f1af7-66c2-d61b-5410-ce57b7db7bff"),
-                (b"svn-revision", b"10"),
-            ),
+            "metadata": {
+                "extra_headers": [
+                    ["svn-repo-uuid", "046f1af7-66c2-d61b-5410-ce57b7db7bff"],
+                    ["svn-revision", 10],
+                ]
+            },
         }
 
         self.revision_with_gpgsig = {
@@ -391,7 +391,7 @@ dg1KdHOa34shrKDaOVzW
             "date": {"timestamp": 1428538899, "offset": 480,},
             "committer": {"name": b"Jiang Xin", "email": b"worldhello.net@gmail.com",},
             "committer_date": {"timestamp": 1428538899, "offset": 480,},
-            "extra_headers": ((b"gpgsig", gpgsig),),
+            "metadata": {"extra_headers": [["gpgsig", gpgsig],],},
             "message": b"""Merge branch 'master' of git://github.com/alexhenrie/git-po
 
 * 'master' of git://github.com/alexhenrie/git-po:
@@ -448,10 +448,12 @@ dg1KdHOa34shrKDaOVzW
                 2015, 7, 12, 15, 10, 30, tzinfo=linus_tz
             ),
             "message": b"Linux 4.2-rc2\n",
-            "extra_headers": (
-                (b"svn-repo-uuid", b"046f1af7-66c2-d61b-5410-ce57b7db7bff"),
-                (b"svn-revision", b"10"),
-            ),
+            "metadata": {
+                "extra_headers": [
+                    ["svn-repo-uuid", "046f1af7-66c2-d61b-5410-ce57b7db7bff"],
+                    ["svn-revision", 10],
+                ]
+            },
         }
 
     def test_revision_identifier(self):
@@ -735,7 +737,7 @@ class SnapshotIdentifier(unittest.TestCase):
             identifiers.identifier_to_str(self.all_types["id"]),
         )
 
-    def test_swhid(self):
+    def test_persistent_identifier(self):
         _snapshot_id = _x("c7c108084bc0bf3d81436bf980b46e98bd338453")
         _release_id = "22ece559cc7cc2364edc5e5593d63ae8bd229f9f"
         _revision_id = "309cf2674ee7a0749978cf8265ab91a60aea0f7d"
@@ -747,7 +749,7 @@ class SnapshotIdentifier(unittest.TestCase):
         _directory = {"id": _directory_id}
         _content = {"sha1_git": _content_id}
 
-        for full_type, _hash, expected_swhid, version, _meta in [
+        for full_type, _hash, expected_persistent_id, version, _meta in [
             (
                 SNAPSHOT,
                 _snapshot_id,
@@ -827,15 +829,17 @@ class SnapshotIdentifier(unittest.TestCase):
             ),
         ]:
             if version:
-                actual_value = identifiers.swhid(
+                actual_value = identifiers.persistent_identifier(
                     full_type, _hash, version, metadata=_meta
                 )
             else:
-                actual_value = identifiers.swhid(full_type, _hash, metadata=_meta)
+                actual_value = identifiers.persistent_identifier(
+                    full_type, _hash, metadata=_meta
+                )
 
-            self.assertEqual(actual_value, expected_swhid)
+            self.assertEqual(actual_value, expected_persistent_id)
 
-    def test_swhid_wrong_input(self):
+    def test_persistent_identifier_wrong_input(self):
         _snapshot_id = "notahash4bc0bf3d81436bf980b46e98bd338453"
         _snapshot = {"id": _snapshot_id}
 
@@ -845,10 +849,10 @@ class SnapshotIdentifier(unittest.TestCase):
             ("foo", ""),
         ]:
             with self.assertRaises(ValidationError):
-                identifiers.swhid(_type, _hash)
+                identifiers.persistent_identifier(_type, _hash)
 
-    def test_parse_swhid(self):
-        for swhid, _type, _version, _hash in [
+    def test_parse_persistent_identifier(self):
+        for pid, _type, _version, _hash in [
             (
                 "swh:1:cnt:94a9ed024d3859793618152ea559a168bbcbb5e2",
                 CONTENT,
@@ -880,17 +884,17 @@ class SnapshotIdentifier(unittest.TestCase):
                 "c7c108084bc0bf3d81436bf980b46e98bd338453",
             ),
         ]:
-            expected_result = SWHID(
+            expected_result = PersistentId(
                 namespace="swh",
                 scheme_version=_version,
                 object_type=_type,
                 object_id=_hash,
                 metadata={},
             )
-            actual_result = identifiers.parse_swhid(swhid)
+            actual_result = identifiers.parse_persistent_identifier(pid)
             self.assertEqual(actual_result, expected_result)
 
-        for swhid, _type, _version, _hash, _metadata in [
+        for pid, _type, _version, _hash, _metadata in [
             (
                 "swh:1:cnt:9c95815d9e9d91b8dae8e05d8bbc696fe19f796b;lines=1-18;origin=https://github.com/python/cpython",  # noqa
                 CONTENT,
@@ -906,28 +910,18 @@ class SnapshotIdentifier(unittest.TestCase):
                 {"origin": "deb://Debian/packages/linuxdoc-tools"},
             ),
         ]:
-            expected_result = SWHID(
+            expected_result = PersistentId(
                 namespace="swh",
                 scheme_version=_version,
                 object_type=_type,
                 object_id=_hash,
                 metadata=_metadata,
             )
-            actual_result = identifiers.parse_swhid(swhid)
+            actual_result = identifiers.parse_persistent_identifier(pid)
             self.assertEqual(actual_result, expected_result)
-            self.assertEqual(
-                expected_result.to_dict(),
-                {
-                    "namespace": "swh",
-                    "scheme_version": _version,
-                    "object_type": _type,
-                    "object_id": _hash,
-                    "metadata": _metadata,
-                },
-            )
 
-    def test_parse_swhid_parsing_error(self):
-        for swhid in [
+    def test_parse_persistent_identifier_parsing_error(self):
+        for pid in [
             ("swh:1:cnt"),
             ("swh:1:"),
             ("swh:"),
@@ -940,7 +934,7 @@ class SnapshotIdentifier(unittest.TestCase):
             ("swh:1:snp:foo"),
         ]:
             with self.assertRaises(ValidationError):
-                identifiers.parse_swhid(swhid)
+                identifiers.parse_persistent_identifier(pid)
 
     def test_persistentid_class_validation_error(self):
         for _ns, _version, _type, _id in [
@@ -950,7 +944,7 @@ class SnapshotIdentifier(unittest.TestCase):
             ("swh", 1, SNAPSHOT, "gh6959356d30f1a4e9b7f6bca59b9a336464c03d"),
         ]:
             with self.assertRaises(ValidationError):
-                SWHID(
+                PersistentId(
                     namespace=_ns,
                     scheme_version=_version,
                     object_type=_type,
@@ -969,108 +963,3 @@ class OriginIdentifier(unittest.TestCase):
             identifiers.origin_identifier(self.origin),
             "b63a575fe3faab7692c9f38fb09d4bb45651bb0f",
         )
-
-
-TS_DICTS = [
-    (
-        {"timestamp": 12345, "offset": 0},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {"timestamp": 12345, "offset": 0, "negative_utc": False},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {"timestamp": 12345, "offset": 0, "negative_utc": False},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {"timestamp": 12345, "offset": 0, "negative_utc": None},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {"timestamp": {"seconds": 12345}, "offset": 0, "negative_utc": None},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": None,
-        },
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 100},
-            "offset": 0,
-            "negative_utc": None,
-        },
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 100},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-    (
-        {"timestamp": 12345, "offset": 0, "negative_utc": True},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": True,
-        },
-    ),
-    (
-        {"timestamp": 12345, "offset": 0, "negative_utc": None},
-        {
-            "timestamp": {"seconds": 12345, "microseconds": 0},
-            "offset": 0,
-            "negative_utc": False,
-        },
-    ),
-]
-
-
-@pytest.mark.parametrize("dict_input,expected", TS_DICTS)
-def test_normalize_timestamp_dict(dict_input, expected):
-    assert normalize_timestamp(dict_input) == expected
-
-
-TS_DICTS_INVALID_TIMESTAMP = [
-    {"timestamp": 1.2, "offset": 0},
-    {"timestamp": "1", "offset": 0},
-    # these below should really also trigger a ValueError...
-    # {"timestamp": {"seconds": "1"}, "offset": 0},
-    # {"timestamp": {"seconds": 1.2}, "offset": 0},
-    # {"timestamp": {"seconds": 1.2}, "offset": 0},
-]
-
-
-@pytest.mark.parametrize("dict_input", TS_DICTS_INVALID_TIMESTAMP)
-def test_normalize_timestamp_dict_invalid_timestamp(dict_input):
-    with pytest.raises(ValueError, match="non-integer timestamp"):
-        normalize_timestamp(dict_input)
